@@ -32,7 +32,7 @@
 /* local prototypes */
 
 static void build_fsaa_translation_table(CtkMultisample *ctk_multisample,
-                                         NVCTRLAttributeValidValuesRec valid);
+                                         CtrlAttributeValidValues valid);
 
 static int map_nv_ctrl_fsaa_value_to_slider(CtkMultisample *ctk_multisample,
                                             int value);
@@ -49,24 +49,23 @@ static GtkWidget *create_fsaa_setting_menu(CtkMultisample *ctk_multisample,
 static void fsaa_setting_checkbox_toggled(GtkWidget *widget,
                                           gpointer user_data);
 
-static void fsaa_setting_menu_changed(GtkObject *object, gpointer user_data);
+static void fsaa_setting_menu_changed(GObject *object, gpointer user_data);
 
-static void fsaa_setting_update_received(GtkObject *object,
-                                         gpointer arg1,
+static void fsaa_setting_update_received(GObject *object, CtrlEvent *event,
                                          gpointer user_data);
 
 static void post_fsaa_value_changed(CtkMultisample *ctk_multisample, gint val);
 
 static void fsaa_value_changed(GtkRange *range, gpointer user_data);
 
-static void fsaa_update_received(GtkObject *object,
-                                 gpointer arg1, gpointer user_data);
+static void fsaa_update_received(GObject *object, CtrlEvent *event,
+                                 gpointer user_data);
 
 static void fxaa_checkbox_toggled(GtkWidget *widget,
                                   gpointer user_data);
 
-static void fxaa_update_received(GtkObject *object,
-                                 gpointer arg1, gpointer user_data);
+static void fxaa_update_received(GObject *object, CtrlEvent *event,
+                                 gpointer user_data);
 
 static void post_fxaa_toggled(CtkMultisample *ctk_multisample, 
                               gboolean enable);
@@ -78,8 +77,7 @@ post_log_aniso_app_override_toggled(CtkMultisample *ctk_multisample,
 static void log_aniso_app_override_toggled(GtkWidget *widget,
                                            gpointer user_data);
 
-static void log_app_override_update_received(GtkObject *object,
-                                             gpointer arg1,
+static void log_app_override_update_received(GObject *object, CtrlEvent *event,
                                              gpointer user_data);
 
 static const gchar *get_log_aniso_name(gint val);
@@ -92,19 +90,9 @@ static void post_log_aniso_value_changed(CtkMultisample *ctk_multisample,
 
 static void log_aniso_value_changed(GtkRange *range, gpointer user_data);
 
-static void log_aniso_range_update_received(GtkObject *object,
-                                            gpointer arg1,
+static void log_aniso_range_update_received(GObject *object, CtrlEvent *event,
                                             gpointer user_data);
 
-static void post_texture_sharpening_toggled(CtkMultisample *ctk_multisample,
-                                            gboolean enabled);
-
-static void texture_sharpening_toggled(GtkWidget *widget, gpointer user_data);
-
-static void texture_sharpening_update_received(GtkObject *object,
-                                               gpointer arg1,
-                                               gpointer user_data);
-
 static void update_fxaa_from_fsaa_change(CtkMultisample *ctk_multisample,
                                          int fsaa_value);
 static void update_fsaa_from_fxaa_change(CtkMultisample *ctk_multisample,
@@ -146,11 +134,6 @@ static const char *__fxaa_enable_help =
 "FXAA disables triple buffering, antialiasing, and other antialiasing "
 "setting methods.";
 
-static const char *__texture_sharpening_help =
-"To improve image quality, select this option "
-"to sharpen textures when running OpenGL applications "
-"with antialiasing enabled.";
-
 
 /*
  * bits indicating which attributes require documenting in the online
@@ -177,7 +160,6 @@ static const char *__texture_sharpening_help =
 #define __FSAA_ENHANCE    (1 << (NV_CTRL_FSAA_MODE_MAX + 2))
 #define __FXAA            (1 << (NV_CTRL_FSAA_MODE_MAX + 3))
 #define __LOG_ANISO_RANGE (1 << (NV_CTRL_FSAA_MODE_MAX + 4))
-#define __TEXTURE_SHARPEN (1 << (NV_CTRL_FSAA_MODE_MAX + 5))
 
 
 #define FRAME_PADDING 5
@@ -216,8 +198,9 @@ GType ctk_multisample_get_type(
  * ctk_multisample_new() - constructor for the Multisample widget
  */
 
-GtkWidget *ctk_multisample_new(NvCtrlAttributeHandle *handle,
-                               CtkConfig *ctk_config, CtkEvent *ctk_event)
+GtkWidget *ctk_multisample_new(CtrlTarget *ctrl_target,
+                               CtkConfig *ctk_config,
+                               CtkEvent *ctk_event)
 {
     GObject *object;
     CtkMultisample *ctk_multisample;
@@ -227,28 +210,28 @@ GtkWidget *ctk_multisample_new(NvCtrlAttributeHandle *handle,
     GtkWidget *check_button;
     GtkWidget *menu;
     GtkWidget *scale;
-    GtkObject *adjustment;
+    GtkAdjustment *adjustment;
     gint min, max;
-    
+
     gint val, app_control, override, enhance, mode, i;
-    
-    NVCTRLAttributeValidValuesRec valid;
+
+    CtrlAttributeValidValues valid;
 
     ReturnStatus ret, ret0;
 
     /* create the new object */
-    
+
     object = g_object_new(CTK_TYPE_MULTISAMPLE, NULL);
 
     ctk_multisample = CTK_MULTISAMPLE(object);
-    ctk_multisample->handle = handle;
+    ctk_multisample->ctrl_target = ctrl_target;
     ctk_multisample->ctk_config = ctk_config;
     ctk_multisample->active_attributes = 0;
 
     gtk_box_set_spacing(GTK_BOX(object), 10);
 
     /* banner */
-    
+
     hbox = gtk_hbox_new(FALSE, 0);
     gtk_box_pack_start(GTK_BOX(object), hbox, FALSE, FALSE, 0);
 
@@ -257,17 +240,18 @@ GtkWidget *ctk_multisample_new(NvCtrlAttributeHandle *handle,
 
     /* FSAA slider */
 
-    ret = NvCtrlGetValidAttributeValues(handle, NV_CTRL_FSAA_MODE, &valid);
-    
+    ret = NvCtrlGetValidAttributeValues(ctrl_target, NV_CTRL_FSAA_MODE,
+                                        &valid);
+
     if (ret == NvCtrlSuccess) {
-        
+
         build_fsaa_translation_table(ctk_multisample, valid);
-        
-        ret = NvCtrlGetAttribute(handle, NV_CTRL_FSAA_MODE, &mode);
-        
+
+        ret = NvCtrlGetAttribute(ctrl_target, NV_CTRL_FSAA_MODE, &mode);
+
         val = map_nv_ctrl_fsaa_value_to_slider(ctk_multisample, mode);
 
-        ret0 = NvCtrlGetAttribute(handle,
+        ret0 = NvCtrlGetAttribute(ctrl_target,
                                   NV_CTRL_FSAA_APPLICATION_CONTROLLED,
                                   &app_control);
         /*
@@ -292,10 +276,10 @@ GtkWidget *ctk_multisample_new(NvCtrlAttributeHandle *handle,
             vbox = gtk_vbox_new(FALSE, 5);
             gtk_container_set_border_width(GTK_CONTAINER(vbox), FRAME_PADDING);
             gtk_container_add(GTK_CONTAINER(frame), vbox);
-            
+
             /* "Application Setting" widget */
-            
-            ret = NvCtrlGetAttribute(ctk_multisample->handle,
+
+            ret = NvCtrlGetAttribute(ctrl_target,
                                      NV_CTRL_FSAA_APPLICATION_ENHANCED,
                                      &enhance);
 
@@ -347,7 +331,8 @@ GtkWidget *ctk_multisample_new(NvCtrlAttributeHandle *handle,
             max = ctk_multisample->fsaa_translation_table_size - 1;
 
             /* create the slider */
-            adjustment = gtk_adjustment_new(val, min, max, 1, 1, 0.0);
+            adjustment = GTK_ADJUSTMENT(gtk_adjustment_new(val, min, max,
+                                                           1, 1, 0.0));
             scale = gtk_hscale_new(GTK_ADJUSTMENT(adjustment));
             gtk_adjustment_set_value(GTK_ADJUSTMENT(adjustment), val);
             
@@ -386,7 +371,7 @@ GtkWidget *ctk_multisample_new(NvCtrlAttributeHandle *handle,
             check_button = gtk_check_button_new_with_label("Enable FXAA");
 
             if (mode == NV_CTRL_FSAA_MODE_NONE) {
-                ret = NvCtrlGetAttribute(handle, NV_CTRL_FXAA, &val);
+                ret = NvCtrlGetAttribute(ctrl_target, NV_CTRL_FXAA, &val);
                 if (val == NV_CTRL_FXAA_ENABLE) {
                     gtk_widget_set_sensitive(GTK_WIDGET(scale), FALSE);
                 }
@@ -414,18 +399,18 @@ GtkWidget *ctk_multisample_new(NvCtrlAttributeHandle *handle,
             ctk_multisample->fxaa_enable_check_button = check_button;
         }
     }
-    
+
     /* Anisotropic filtering slider */
 
-    ret = NvCtrlGetValidAttributeValues(handle, NV_CTRL_LOG_ANISO, &valid);
-    
+    ret = NvCtrlGetValidAttributeValues(ctrl_target, NV_CTRL_LOG_ANISO, &valid);
+
     ctk_multisample->log_aniso_scale = NULL;
 
     if (ret == NvCtrlSuccess) {
-        
-        ret = NvCtrlGetAttribute(handle, NV_CTRL_LOG_ANISO, &val);
 
-        ret0 = NvCtrlGetAttribute(handle,
+        ret = NvCtrlGetAttribute(ctrl_target, NV_CTRL_LOG_ANISO, &val);
+
+        ret0 = NvCtrlGetAttribute(ctrl_target,
                                   NV_CTRL_LOG_ANISO_APPLICATION_CONTROLLED,
                                   &app_control);
         /*
@@ -438,7 +423,7 @@ GtkWidget *ctk_multisample_new(NvCtrlAttributeHandle *handle,
         override = !app_control;
         
         if ((ret == NvCtrlSuccess) && (ret0 == NvCtrlSuccess) &&
-            (valid.type == ATTRIBUTE_TYPE_RANGE)) {
+            (valid.valid_type == CTRL_ATTRIBUTE_VALID_TYPE_RANGE)) {
             
             /* create "Anisotropic Filtering" frame */
             
@@ -478,11 +463,12 @@ GtkWidget *ctk_multisample_new(NvCtrlAttributeHandle *handle,
             
             /* Aniso scale */
 
-            min = valid.u.range.min;
-            max = valid.u.range.max;
+            min = valid.range.min;
+            max = valid.range.max;
 
             /* create the slider */
-            adjustment = gtk_adjustment_new(val, min, max, 1, 1, 0.0);
+            adjustment = GTK_ADJUSTMENT(gtk_adjustment_new(val, min, max,
+                                                           1, 1, 0.0));
             scale = gtk_hscale_new(GTK_ADJUSTMENT(adjustment));
             gtk_adjustment_set_value(GTK_ADJUSTMENT(adjustment), val);
 
@@ -513,52 +499,6 @@ GtkWidget *ctk_multisample_new(NvCtrlAttributeHandle *handle,
         }
     }
     
-    /*
-     * Texture sharpen
-     *
-     * If one of the supported multisample modes was enabled by the
-     * user, this check button controls texture sharpening.
-     */
-    
-    ret = NvCtrlGetAttribute(handle, NV_CTRL_TEXTURE_SHARPEN, &val);
-
-    if (ret == NvCtrlSuccess) {
-        
-        /* create "TextureSharpening" frame */
-
-        frame = gtk_frame_new("Texture Quality");
-        gtk_box_pack_start(GTK_BOX(object), frame, FALSE, FALSE, 0);
-        
-        /* create the vbox to store the widgets inside the frame */
-
-        vbox = gtk_vbox_new(FALSE, 5);
-        gtk_container_set_border_width(GTK_CONTAINER(vbox), FRAME_PADDING);
-        gtk_container_add(GTK_CONTAINER(frame), vbox);
-        
-        /* "Texture Sharpening" checkbox */
-        
-        check_button = gtk_check_button_new_with_label("Texture Sharpening");
-        
-        gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(check_button), val);
-        
-        gtk_box_pack_start(GTK_BOX(vbox), check_button, FALSE, FALSE, 0);
-        
-        g_signal_connect(G_OBJECT(check_button), "toggled",
-                         G_CALLBACK(texture_sharpening_toggled),
-                         (gpointer) ctk_multisample);
-        
-        g_signal_connect(G_OBJECT(ctk_event),
-                         CTK_EVENT_NAME(NV_CTRL_TEXTURE_SHARPEN),
-                         G_CALLBACK(texture_sharpening_update_received),
-                         (gpointer) ctk_multisample);
-        
-        ctk_config_set_tooltip(ctk_config, check_button,
-                               __texture_sharpening_help);
-        
-        ctk_multisample->active_attributes |= __TEXTURE_SHARPEN;
-        ctk_multisample->texture_sharpening_button = check_button;
-    }
-    
     /* if nothing is available, teardown this object and return NULL */
 
     if (!ctk_multisample->active_attributes) {
@@ -584,21 +524,23 @@ GtkWidget *ctk_multisample_new(NvCtrlAttributeHandle *handle,
  */
 
 static void build_fsaa_translation_table(CtkMultisample *ctk_multisample,
-                                         NVCTRLAttributeValidValuesRec valid)
+                                         CtrlAttributeValidValues valid)
 {
     gint i, n = 0;
     gint index_8xs = -1;
     gint index_16x = -1;
     gint index_32x = -1;
     gint index_32xs = -1;
-    gint mask = valid.u.bits.ints;
+    gint mask = valid.allowed_ints;
 
     ctk_multisample->fsaa_translation_table_size = 0;
 
     memset(ctk_multisample->fsaa_translation_table, 0,
            sizeof(gint) * (NV_CTRL_FSAA_MODE_MAX + 1));
 
-    if (valid.type != ATTRIBUTE_TYPE_INT_BITS) return;
+    if (valid.valid_type != CTRL_ATTRIBUTE_VALID_TYPE_INT_BITS) {
+        return;
+    }
     
     for (i = 0; i <= NV_CTRL_FSAA_MODE_MAX; i++) {
         if (mask & (1 << i)) {
@@ -782,18 +724,19 @@ static void post_fsaa_setting_changed(CtkMultisample *ctk_multisample,
 static void update_fsaa_setting(CtkMultisample *ctk_multisample,
                                 gboolean override, gboolean enhance)
 {
+    CtrlTarget *ctrl_target = ctk_multisample->ctrl_target;
     GtkRange *range = GTK_RANGE(ctk_multisample->fsaa_scale);
 
-    NvCtrlSetAttribute(ctk_multisample->handle,
+    NvCtrlSetAttribute(ctrl_target,
                        NV_CTRL_FSAA_APPLICATION_CONTROLLED, !override);
-    
+
     if (ctk_multisample->active_attributes & __FSAA_ENHANCE) {
-        NvCtrlSetAttribute(ctk_multisample->handle,
+        NvCtrlSetAttribute(ctrl_target,
                            NV_CTRL_FSAA_APPLICATION_ENHANCED, enhance);
     }
 
     if (!override) {
-        NvCtrlSetAttribute(ctk_multisample->handle,
+        NvCtrlSetAttribute(ctrl_target,
                            NV_CTRL_FSAA_MODE, NV_CTRL_FSAA_MODE_NONE);
 
         g_signal_handlers_block_by_func(G_OBJECT(range),
@@ -842,7 +785,7 @@ static void fsaa_setting_checkbox_toggled(GtkWidget *widget,
  * the fsaa slider.
  */
 
-static void fsaa_setting_menu_changed(GtkObject *object, gpointer user_data)
+static void fsaa_setting_menu_changed(GObject *object, gpointer user_data)
 {
     CtkMultisample *ctk_multisample = CTK_MULTISAMPLE(user_data);
     gint idx;
@@ -874,12 +817,12 @@ static void fsaa_setting_menu_changed(GtkObject *object, gpointer user_data)
  * NV-CONTROL client.
  */
 
-static void fsaa_setting_update_received(GtkObject *object,
-                                         gpointer arg1,
+static void fsaa_setting_update_received(GObject *object,
+                                         CtrlEvent *event,
                                          gpointer user_data)
 {
-    CtkEventStruct *event_struct = (CtkEventStruct *) arg1;
     CtkMultisample *ctk_multisample = CTK_MULTISAMPLE(user_data);
+    CtrlTarget *ctrl_target = ctk_multisample->ctrl_target;
     gint idx;
     gboolean override;
     gboolean enhance = FALSE;
@@ -887,15 +830,18 @@ static void fsaa_setting_update_received(GtkObject *object,
     gint val;
     ReturnStatus ret;
 
+    if (event->type != CTRL_EVENT_TYPE_INTEGER_ATTRIBUTE) {
+        return;
+    }
 
-    switch (event_struct->attribute) {
+    switch (event->int_attr.attribute) {
     case NV_CTRL_FSAA_APPLICATION_CONTROLLED:
-        override = !event_struct->value;
+        override = !event->int_attr.value;
 
         if (!override) {
             idx = 0;
         } else if (ctk_multisample->active_attributes & __FSAA_ENHANCE) {
-            ret = NvCtrlGetAttribute(ctk_multisample->handle,
+            ret = NvCtrlGetAttribute(ctrl_target,
                                      NV_CTRL_FSAA_APPLICATION_ENHANCED,
                                      &val);
             if (ret == NvCtrlSuccess) {
@@ -911,9 +857,9 @@ static void fsaa_setting_update_received(GtkObject *object,
         break;
 
     case NV_CTRL_FSAA_APPLICATION_ENHANCED:
-        enhance = event_struct->value;
+        enhance = event->int_attr.value;
 
-        ret = NvCtrlGetAttribute(ctk_multisample->handle,
+        ret = NvCtrlGetAttribute(ctrl_target,
                                  NV_CTRL_FSAA_APPLICATION_CONTROLLED,
                                  &val);
         if (ret == NvCtrlSuccess) {
@@ -1079,20 +1025,19 @@ static void update_fsaa_from_fxaa_change (CtkMultisample *ctk_multisample,
 
 static void fsaa_value_changed(GtkRange *range, gpointer user_data)
 {
-    CtkMultisample *ctk_multisample;
+    CtkMultisample *ctk_multisample = CTK_MULTISAMPLE(user_data); 
+    CtrlTarget *ctrl_target = ctk_multisample->ctrl_target;
     gint val;
 
-    ctk_multisample = CTK_MULTISAMPLE(user_data); 
-
     val = gtk_range_get_value(range);
     if (val > NV_CTRL_FSAA_MODE_MAX) val = NV_CTRL_FSAA_MODE_MAX;
     if (val < 0) val = 0;
     val = ctk_multisample->fsaa_translation_table[val];
 
-    NvCtrlSetAttribute(ctk_multisample->handle, NV_CTRL_FSAA_MODE, val);
+    NvCtrlSetAttribute(ctrl_target, NV_CTRL_FSAA_MODE, val);
 
     update_fxaa_from_fsaa_change(ctk_multisample, val);
-    
+
     post_fsaa_value_changed(ctk_multisample, val);
 
 } /* fsaa_value_changed() */
@@ -1106,11 +1051,12 @@ static void fxaa_checkbox_toggled(GtkWidget *widget,
                                   gpointer user_data)
 {
     CtkMultisample *ctk_multisample = CTK_MULTISAMPLE(user_data);
+    CtrlTarget *ctrl_target = ctk_multisample->ctrl_target;
     gboolean enabled;
 
     enabled = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget));
 
-    NvCtrlSetAttribute(ctk_multisample->handle, NV_CTRL_FXAA, enabled);
+    NvCtrlSetAttribute(ctrl_target, NV_CTRL_FXAA, enabled);
 
     update_fsaa_from_fxaa_change(ctk_multisample, enabled);
 
@@ -1125,14 +1071,20 @@ static void fxaa_checkbox_toggled(GtkWidget *widget,
  * client.
  */
 
-static void fxaa_update_received(GtkObject *object,
-                                 gpointer arg1, gpointer user_data)
+static void fxaa_update_received(GObject *object,
+                                 CtrlEvent *event,
+                                 gpointer user_data)
 {
-    CtkEventStruct *event_struct = (CtkEventStruct *) arg1;
     CtkMultisample *ctk_multisample = CTK_MULTISAMPLE(user_data);
-    gboolean fxaa_value = event_struct->value;
+    gboolean fxaa_value;
     GtkWidget *check_button = ctk_multisample->fxaa_enable_check_button;
 
+    if (event->type != CTRL_EVENT_TYPE_INTEGER_ATTRIBUTE) {
+        return;
+    }
+
+    fxaa_value = event->int_attr.value;
+
     g_signal_handlers_block_by_func(G_OBJECT(check_button),
                                     G_CALLBACK(fxaa_checkbox_toggled),
                                     (gpointer) ctk_multisample);
@@ -1172,20 +1124,23 @@ post_fxaa_toggled(CtkMultisample *ctk_multisample, gboolean enable)
  * client.
  */
 
-static void fsaa_update_received(GtkObject *object,
-                                 gpointer arg1, gpointer user_data)
+static void fsaa_update_received(GObject *object,
+                                 CtrlEvent *event,
+                                 gpointer user_data)
 {
-    CtkEventStruct *event_struct;
     CtkMultisample *ctk_multisample;
     GtkRange *range;
     gint val;
 
-    event_struct = (CtkEventStruct *) arg1;
+    if (event->type != CTRL_EVENT_TYPE_INTEGER_ATTRIBUTE) {
+        return;
+    }
+
     ctk_multisample = CTK_MULTISAMPLE(user_data);
     range = GTK_RANGE(ctk_multisample->fsaa_scale);
 
     val = map_nv_ctrl_fsaa_value_to_slider(ctk_multisample,
-                                           event_struct->value);
+                                           event->int_attr.value);
 
     g_signal_handlers_block_by_func(G_OBJECT(range),
                                     G_CALLBACK(fsaa_value_changed),
@@ -1193,13 +1148,13 @@ static void fsaa_update_received(GtkObject *object,
     
     gtk_range_set_value(range, val);
 
-    update_fxaa_from_fsaa_change(ctk_multisample, event_struct->value);
+    update_fxaa_from_fsaa_change(ctk_multisample, event->int_attr.value);
     
     g_signal_handlers_unblock_by_func(G_OBJECT(range),
                                       G_CALLBACK(fsaa_value_changed),
                                       (gpointer) ctk_multisample);
 
-    post_fsaa_value_changed(ctk_multisample, event_struct->value);
+    post_fsaa_value_changed(ctk_multisample, event->int_attr.value);
 
 } /* fsaa_update_received() */
 
@@ -1240,15 +1195,16 @@ static void log_aniso_app_override_toggled(GtkWidget *widget,
                                            gpointer user_data)
 {
     CtkMultisample *ctk_multisample = CTK_MULTISAMPLE(user_data);
+    CtrlTarget *ctrl_target = ctk_multisample->ctrl_target;
     GtkRange *range = GTK_RANGE(ctk_multisample->log_aniso_scale);
     gboolean override;
 
     override = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget));
 
-    NvCtrlSetAttribute(ctk_multisample->handle,
+    NvCtrlSetAttribute(ctrl_target,
                        NV_CTRL_LOG_ANISO_APPLICATION_CONTROLLED, !override);
     if (!override) {
-        NvCtrlSetAttribute(ctk_multisample->handle,
+        NvCtrlSetAttribute(ctrl_target,
                            NV_CTRL_LOG_ANISO, 0 /* default(?) */);
 
         g_signal_handlers_block_by_func(G_OBJECT(range),
@@ -1274,14 +1230,20 @@ static void log_aniso_app_override_toggled(GtkWidget *widget,
  * another NV-CONTROL client.
  */
 
-static void log_app_override_update_received(GtkObject *object,
-                                             gpointer arg1, gpointer user_data)
+static void log_app_override_update_received(GObject *object,
+                                             CtrlEvent *event,
+                                             gpointer user_data)
 {
-    CtkEventStruct *event_struct = (CtkEventStruct *) arg1;
     CtkMultisample *ctk_multisample = CTK_MULTISAMPLE(user_data);
-    gboolean override = !event_struct->value;
+    gboolean override;
     GtkWidget *check_button;
 
+    if (event->type != CTRL_EVENT_TYPE_INTEGER_ATTRIBUTE) {
+        return;
+    }
+
+    override = !event->int_attr.value;
+
     check_button = ctk_multisample->log_aniso_app_override_check_button;
 
     g_signal_handlers_block_by_func(G_OBJECT(check_button),
@@ -1342,10 +1304,15 @@ static gchar *format_log_aniso_value(GtkScale *scale, gdouble arg1,
 static void post_log_aniso_value_changed(CtkMultisample *ctk_multisample,
                                          gint val)
 {
-    ctk_config_statusbar_message(ctk_multisample->ctk_config,
-                                 "Anisotropic Filtering set to %s.",
-                                 get_log_aniso_name(val));
-    
+    gboolean override = gtk_toggle_button_get_active(
+        GTK_TOGGLE_BUTTON(ctk_multisample->log_aniso_app_override_check_button));
+
+    if (override) {
+        ctk_config_statusbar_message(ctk_multisample->ctk_config,
+                                     "Anisotropic Filtering set to %s.",
+                                     get_log_aniso_name(val));
+    }
+
 } /* post_log_aniso_value_changed() */
 
 
@@ -1357,15 +1324,14 @@ static void post_log_aniso_value_changed(CtkMultisample *ctk_multisample,
 
 static void log_aniso_value_changed(GtkRange *range, gpointer user_data)
 {
-    CtkMultisample *ctk_multisample;
+    CtkMultisample *ctk_multisample = CTK_MULTISAMPLE(user_data);
+    CtrlTarget *ctrl_target = ctk_multisample->ctrl_target;
     gint val;
 
-    ctk_multisample = CTK_MULTISAMPLE(user_data);
-
     val = gtk_range_get_value(range);
 
-    NvCtrlSetAttribute(ctk_multisample->handle, NV_CTRL_LOG_ANISO, val);
-    
+    NvCtrlSetAttribute(ctrl_target, NV_CTRL_LOG_ANISO, val);
+
     post_log_aniso_value_changed(ctk_multisample, val);
 
 } /* log_aniso_value_changed() */
@@ -1378,14 +1344,17 @@ static void log_aniso_value_changed(GtkRange *range, gpointer user_data)
  * client.
  */
 
-static void log_aniso_range_update_received(GtkObject *object,
-                                            gpointer arg1, gpointer user_data)
+static void log_aniso_range_update_received(GObject *object,
+                                            CtrlEvent *event,
+                                            gpointer user_data)
 {
-    CtkEventStruct *event_struct;
     CtkMultisample *ctk_multisample;
     GtkRange *range;
 
-    event_struct = (CtkEventStruct *) arg1;
+    if (event->type != CTRL_EVENT_TYPE_INTEGER_ATTRIBUTE) {
+        return;
+    }
+
     ctk_multisample = CTK_MULTISAMPLE(user_data);
     range = GTK_RANGE(ctk_multisample->log_aniso_scale);
 
@@ -1393,9 +1362,9 @@ static void log_aniso_range_update_received(GtkObject *object,
                                     G_CALLBACK(log_aniso_value_changed),
                                     (gpointer) ctk_multisample);
     
-    gtk_range_set_value(range, event_struct->value);
+    gtk_range_set_value(range, event->int_attr.value);
     
-    post_log_aniso_value_changed(ctk_multisample, event_struct->value);
+    post_log_aniso_value_changed(ctk_multisample, event->int_attr.value);
 
     g_signal_handlers_unblock_by_func(G_OBJECT(range),
                                       G_CALLBACK(log_aniso_value_changed),
@@ -1405,82 +1374,6 @@ static void log_aniso_range_update_received(GtkObject *object,
 
 
 
-/*
- * post_texture_sharpening_toggled() - helper function for
- * texture_sharpening_toggled() and
- * texture_sharpening_update_received(); this does whatever work is
- * necessary after the texture sharpening button has been toggled --
- * currently, just post a statusbar message.
- */
-
-static void post_texture_sharpening_toggled(CtkMultisample *ctk_multisample,
-                                            gboolean enabled)
-{
-    ctk_config_statusbar_message(ctk_multisample->ctk_config,
-                                 "Texture sharpening %s.",
-                                 enabled ? "enabled" : "disabled");
-    
-} /* post_texture_sharpening_toggled() */
-
-
-
-/*
- * texture_sharpening_toggled() - callback for the "toggled" signal
- * from the texture sharpening check button.
- */
-
-static void texture_sharpening_toggled(GtkWidget *widget, gpointer user_data)
-{
-    CtkMultisample *ctk_multisample;
-    gboolean enabled;
-
-    ctk_multisample = CTK_MULTISAMPLE(user_data);
-
-    enabled = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget));
-
-    NvCtrlSetAttribute(ctk_multisample->handle, NV_CTRL_TEXTURE_SHARPEN,
-                       enabled);
-
-    post_texture_sharpening_toggled(ctk_multisample, enabled);
-    
-} /* texture_sharpening_toggled() */
-
-
-
-/*
- * texture_sharpening_update_received() - callback function for when
- * the NV_CTRL_TEXTURE_SHARPEN attribute is changed by another
- * NV-CONTROL client.
- */
-
-static void texture_sharpening_update_received(GtkObject *object,
-                                               gpointer arg1,
-                                               gpointer user_data)
-{
-    CtkEventStruct *event_struct;
-    CtkMultisample *ctk_multisample;
-    GtkToggleButton *button;
-    
-    event_struct = (CtkEventStruct *) arg1;
-    ctk_multisample = CTK_MULTISAMPLE(user_data);
-    button = GTK_TOGGLE_BUTTON(ctk_multisample->texture_sharpening_button);
-
-    g_signal_handlers_block_by_func(G_OBJECT(button),
-                                    G_CALLBACK(texture_sharpening_toggled),
-                                    (gpointer) ctk_multisample);
-
-    gtk_toggle_button_set_active(button, event_struct->value);
-    
-    post_texture_sharpening_toggled(ctk_multisample, event_struct->value);
-
-    g_signal_handlers_unblock_by_func(G_OBJECT(button), 
-                                      G_CALLBACK(texture_sharpening_toggled),
-                                      (gpointer) ctk_multisample);
-    
-} /* texture_sharpening_update_received() */
-
-
-
 /*
  * ctk_multisample_create_help() - create a GtkTextBuffer describing
  * the available image quality options.
@@ -1674,11 +1567,6 @@ GtkTextBuffer *ctk_multisample_create_help(GtkTextTagTable *table,
         ctk_help_para(b, &i, "%s", __aniso_slider_help);
     }
 
-    if (ctk_multisample->active_attributes & __TEXTURE_SHARPEN) {
-        ctk_help_heading(b, &i, "Texture Sharpening");
-        ctk_help_para(b, &i, "%s", __texture_sharpening_help);
-    }
-
     ctk_help_finish(b);
 
     return b;
