@@ -99,7 +99,7 @@ static const NVGetoptOption __options[] = {
       "an assignment to something other than an X screen.  A target "
       "specification is contained within brackets and consists of a target "
       "type name, a colon, and the target id.  The target type name can be "
-      "one of ^\"screen\", \"gpu\", \"framelock\", \"vcs\", \"gvi\", \"fan\", "
+      "one of ^\"screen\", \"gpu\", \"framelock\", \"fan\", "
       "\"thermalsensor\", \"svp\",^ or ^\"dpy\";^ the target id is the index "
       "into the list of targets (for that target type).  The target "
       "specification can be used in {DISPLAY} wherever an X screen can be used, "
@@ -111,7 +111,7 @@ static const NVGetoptOption __options[] = {
       "for information on which target types can be used with which "
       "attributes.  See the output of\n"
       "\n"
-      TAB " nvidia-settings -q screens -q gpus -q framelocks -q vcs -q gvis "
+      TAB " nvidia-settings -q screens -q gpus -q framelocks "
       "-q fans -q thermalsensors -q svps -q dpys \n"
       "\n"
       "for lists of targets for each target type.\n"
@@ -135,9 +135,9 @@ static const NVGetoptOption __options[] = {
       "This queries the current value of the attribute {attribute name} on the "
       "X Display {DISPLAY}.  The syntax is the same as that for the "
       "^'--assign'^ option, without '=^{value}'^; specify ^'-q screens', "
-      "'-q gpus', '-q framelocks', '-q vcs', '-q gvis', '-q fans'^, "
+      "'-q gpus', '-q framelocks', '-q fans'^, "
       "'-q thermalsensors', '-q svps', or '-q dpys' to query a list of X "
-      "screens, GPUs, Frame Lock devices, Visual Computing Systems, SDI Input "
+      "screens, GPUs, Frame Lock devices, Visual Computing Systems, "
       "Devices, Fans, Thermal Sensors, 3D Vision Pro Transceivers, or Display "
       "Devices, respectively, that are present on the X Display {DISPLAY}.  "
       "Specify ^'-q all'^ to query all attributes." },
